<?php

global $config;

function constructRequest($json) {
    global $config;

    // Take our data and construct a valid ICE XML Request body (technically vMR)
    $parsed                         = [];
    $parsed['contraindications']    = [];
    $request                        = $json['soapenv_Body']['ns_GetForecastRequest'];

    // Get demographic data and evaluation time
    $person                                 = $request['ns_personDetails'];
    $parsed['evaluation']                   = isset($person['@attributes']['evaluationDate']) ? $person['@attributes']['evaluationDate'] : date('Ymd');
    $parsed['person']['id']                 = isset($person['@attributes']['personId']) ? $person['@attributes']['personId'] : "";
    $parsed['person']['dob']                = $person['ns_dateOfBirth'];
    $parsed['person']['gender']['code']     = isset($person['ns_gender']) ? $person['ns_gender'] : "UN";

    // Decide full gender display name based on code
    $genderFull = "";
    switch($parsed['person']['gender']['code']) {
        case "M":
            $genderFull = "Male";
            break;
        case "F":
            $genderFull = "Female";
            break;
        default:
            $genderFull = "Undifferentiated";
            $parsed['person']['gender']['code'] = "UN";
            break;    
    }

    $parsed['person']['gender']['display']  = $genderFull;

    // Loop through vaccination events
    if (isset($person['ns_vaccination'])) {
        $vaccEvents           = $person['ns_vaccination'];

        // get the names of the keys in the array
        $keys = array_keys($vaccEvents);
        // check if the first key is an integer (list) or a string (map)
        $isList = is_int($keys[0]);

        if (!$isList) {
            
            $newVacc        = [];
            $newVacc[]      = $vaccEvents;
            $vaccEvents     = $newVacc;
        }

        foreach ($vaccEvents as $vaccination) {

            // Bug Fix: STC handles single digit CVX codes, prepending them with a zero - ICE does not
            // If ns_vaccCode is a single digit, prepend it with a zero
            if (strlen($vaccination['ns_vaccCode']) == 1) {
                $vaccination['ns_vaccCode'] = "0" . $vaccination['ns_vaccCode'];
            }

            $vacc = [];
            $vacc['date']               = $vaccination['ns_vaccDate'];
            $vacc['code']               = $vaccination['@attributes']['codeType']; // ICE only accepts CVX - STC *can* support CPT, but OpenCast doesn't implement a CPT to CVX map at this time
            $vacc['cvx']                = $vaccination['ns_vaccCode'];
            $vacc['dose']               = isset($vaccination['ns_doseSize']) ? $vaccination['ns_doseSize'] : "1"; // This is not used by ICE 

            // Vaccines that are marked as compromised will have a compromisedCode - ICE only accepts this if
            // Dose Override is enabled, so we'll check for that when building the payload
            // if it isn't enabled, we'll drop compromised vaccines from the input
            $vacc['isValid']            = isset($vaccination['ns_compromisedCode']) ? (is_array($vaccination['ns_compromisedCode']) ? true : false) : true;
            
            $parsed['vaccinations'][]   = $vacc;
        }
    }

    // TODO: ICE does *not* support contraindications like STC. STC expects a CVX code, but ICE expects documentation of disease
    // as such, we don't do anything with STC contraindications at this time

    // Loop through contraidications, but the map can be weird
    if (isset($person['ns_contraindication'])) {
        $contraindications      = $person['ns_contraindication'];
        $odd                    = false;
        $contra                 = [];
        $singleMode             = true;

        foreach ($contraindications as $contraindication) {
            // Check if $contraindication['@attributes'] exists
            if (isset($contraindication['@attributes'])) {
                $singleMode = false;
            }
            
            if ($singleMode) {
                if (!$odd) { 
                    $contra = [];
                    $contra['code']             = $contraindication['codeType']; 
                } else {
                    $contra['cvx']              = $contraindication;
                    $parsed['contraindications'][] = $contra;
                }

                $odd = !$odd;

            } else {
                $contra = [];
                $contra['code']                     = $contraindication['@attributes']['codeType'];
                $contra['cvx']                      = $contraindication['ns_vaccCode'];
                $parsed['contraindications'][]      = $contra;
            }
        }
    }

    $payload = buildPayload($parsed);
    $message = buildMessage($payload, $parsed['evaluation']);

    $return                 = [];

    $return['payload']      = $message;

    // Future info contains the evaluation date and contraindications that are not included in the payload, but
    // may be needed as we construct our STC response
    $return['futureInfo']   = ['evaluationDate' => $parsed['evaluation'], 'personId' => $parsed['person']['id'], 'contra' => $parsed['contraindications']];

    return $return;
}

function buildPayload($map) {
    global $config;

    // Create a unique GUID for the request
    $guid = uniqid();

    // Create a unique extension for the request, limited to 3 digits
    $extension = substr($guid, -3);

    // Build XML header and root
    $xml = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>
    <ns4:cdsInput xmlns:ns2=\"org.opencds\" xmlns:ns3=\"org.opencds.vmr.v1_0.schema.vmr\"
    xmlns:ns4=\"org.opencds.vmr.v1_0.schema.cdsinput\" xmlns:ns5=\"org.opencds.vmr.v1_0.schema.cdsoutput\">
    <templateId root=\"2.16.840.1.113883.3.795.11.1.1\"/>
    <cdsContext>
    <cdsSystemUserPreferredLanguage code=\"en\" codeSystem=\"2.16.840.1.113883.6.99\"
    displayName=\"English\"/>
    </cdsContext>
    <vmrInput>
    <templateId root=\"2.16.840.1.113883.3.795.11.1.1\"/>
    <patient>
    <templateId root=\"2.16.840.1.113883.3.795.11.2.1.1\"/>
    <id root=\"{$guid}\" extension=\"{$extension}\"/>
    <demographics>
    <birthTime value=\"{$map['person']['dob']}\"/>
    <gender code=\"{$map['person']['gender']['code']}\" codeSystem=\"2.16.840.1.113883.5.1\" displayName=\"{$map['person']['gender']['display']}\"
    originalText=\"{$map['person']['gender']['code']}\"/>
    </demographics>
    <clinicalStatements>
    <substanceAdministrationEvents>";

    // Build vaccination events
    $vaccinations = "";

    if (isset($map['vaccinations'])) {
        foreach ($map['vaccinations'] as $vaccination) {
            // create a unique identifier for each vaccination
            $unique_id = uniqid();
            
            // Add R- for the root ID
            $unique_id_root = "R-" . $unique_id;

            // Handle doseOverride feature, if enabled
            $compromisedXML = "";
            if ($config['ice']['doseOverride']) {
                $validString    = $vaccination['isValid'] ? "true" : "false";
                $compromisedXML = "<isValid value=\"{$validString}\"/>";
            } else if (!$vaccination['isValid']) {
                // If the vaccination isn't valid, and doseOverride is disabled, do _not_ write this vaccination to the payload
                continue;
            }

            // Build the XML
            $vaccinations .= "<substanceAdministrationEvent><templateId root=\"2.16.840.1.113883.3.795.11.9.1.1\"/><id root=\"{$unique_id_root}\"/><substanceAdministrationGeneralPurpose code=\"384810002\" codeSystem=\"2.16.840.1.113883.6.5\"/><substance><id root=\"{$unique_id}\"/><substanceCode code=\"{$vaccination['cvx']}\" codeSystem=\"2.16.840.1.113883.12.292\"/></substance><administrationTimeInterval low=\"{$vaccination['date']}\" high=\"{$vaccination['date']}\"/>{$compromisedXML}</substanceAdministrationEvent>";
        }
    }

    $xml = $xml . $vaccinations;

    // Build footer
    $xml = $xml . "</substanceAdministrationEvents>
    </clinicalStatements>
    </patient>
    </vmrInput>
    </ns4:cdsInput>";

    // Remove linebreaks and tabs
    $xml = str_replace(array("\n", "\r", "\t"), '', $xml);

    // Base64 encode the XML
    $xml = base64_encode($xml);

    return $xml;

}

function buildMessage($encodedPayload, $requestTime) {

    // Convert request time from YYYYMMDD to YYYY-MM-DD
    $requestTime = substr($requestTime, 0, 4) . "-" . substr($requestTime, 4, 2) . "-" . substr($requestTime, 6, 2);

    // Encloses the vMR in the correct envelope
    $message = "<?xml version=\"1.0\" encoding=\"utf-8\"?>
    <S:Envelope xmlns:S=\"http://www.w3.org/2003/05/soap-envelope\">
      <S:Body>
        <ns2:evaluateAtSpecifiedTime xmlns:ns2=\"http://www.omg.org/spec/CDSS/201105/dss\">
          <interactionId scopingEntityId=\"gov.nyc.health\" interactionId=\"123456\"/>
          <specifiedTime>{$requestTime}</specifiedTime>
          <evaluationRequest clientLanguage=\"\" clientTimeZoneOffset=\"\">
            <kmEvaluationRequest>
              <kmId scopingEntityId=\"org.nyc.cir\" businessId=\"ICE\" version=\"1.0.0\"/>
            </kmEvaluationRequest>
            <dataRequirementItemData>
              <driId itemId=\"cdsPayload\">
                <containingEntityId scopingEntityId=\"gov.nyc.health\" businessId=\"ICEData\" version=\"1.0.0.0\"/>
              </driId>
              <data>
                <informationModelSSId scopingEntityId=\"org.opencds.vmr\" businessId=\"VMR\" version=\"1.0\"/>
    <base64EncodedPayload>{$encodedPayload}</base64EncodedPayload>
    </data>
            </dataRequirementItemData>
          </evaluationRequest>
        </ns2:evaluateAtSpecifiedTime>
      </S:Body>
    </S:Envelope>";

    // Remove tabs from the message
    $message = str_replace("\t", '', $message);

    return $message;
}